"""Settings for dj-brevo.

Access django settings via the `brevo_settings`

    from dj_brevo.settings import brevo_settings
    api_key = brevo_settings.API_KEY

User configuration is read from Django settings:

    # settings.py
    DJ_BREVO = {
        "API_KEY": "api_key_xxx",
    }
"""

from dataclasses import dataclass, field
from typing import Any

from django.conf import settings

DEFAULTS: dict[str, Any] = {
    "API_KEY": None,
    "DEFAULT_FROM_EMAIL": None,
    "TIMEOUT": 10,
    "API_BASE_URL": "https://api.brevo.com/v3",
    "SANDBOX": False,
    "AUTO_SYNC": True,
}


@dataclass
class BrevoSettings:
    """Type-safe access to DJ_BREVO settings."""

    _user_settings: dict[str, Any] = field(default_factory=dict)

    def __post_init__(self) -> None:
        self._user_settings = getattr(settings, "DJ_BREVO", {})

    def __getattr__(self, name: str) -> Any:
        if name.startswith("_"):
            raise AttributeError(name)

        if name not in DEFAULTS:
            raise AttributeError(f"Invalid setting: {name!r}")

        # user setting takes priority, otherwise use default
        if name in self._user_settings:
            return self._user_settings[name]
        return DEFAULTS[name]


brevo_settings = BrevoSettings()



"""Tests for signal handlers."""

from unittest.mock import patch

import pytest

from dj_brevo.models import BrevoAttribute, BrevoList


@pytest.mark.django_db
class TestBrevoListSignal:
    """Tests for BrevoList post_save signal."""

    def test_auto_sync_disabled_skips_api_call(self, settings) -> None:
        """Should not call API when AUTO_SYNC is False."""
        settings.DJ_BREVO["AUTO_SYNC"] = False  # Explicitly set

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            BrevoList.objects.create(name="Test List", folder_id=1)

            mock_client.assert_not_called()

    def test_auto_sync_creates_list_in_brevo(self, settings) -> None:
        """Should call create_list when brevo_id is None."""
        settings.DJ_BREVO["AUTO_SYNC"] = True

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            mock_instance = mock_client.return_value
            mock_instance.create_list.return_value = {"id": 12345}

            brevo_list = BrevoList.objects.create(name="Test List", folder_id=1)

            mock_instance.create_list.assert_called_once_with(
                name="Test List",
                folder_id=1,
            )

            # Refresh from DB to get updated values
            brevo_list.refresh_from_db()
            assert brevo_list.brevo_id == 12345
            assert brevo_list.synced_at is not None

    def test_auto_sync_updates_list_in_brevo(self, settings) -> None:
        """Should call update_list when brevo_id exists."""
        settings.DJ_BREVO["AUTO_SYNC"] = True

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            mock_instance = mock_client.return_value
            mock_instance.create_list.return_value = {"id": 12345}

            # Create first (this will set brevo_id)
            brevo_list = BrevoList.objects.create(name="Test List", folder_id=1)

            # Reset mock to check update call
            mock_instance.reset_mock()

            # Update the list
            brevo_list.name = "Updated Name"
            brevo_list.save()

            mock_instance.update_list.assert_called_once_with(
                list_id=12345,
                name="Updated Name",
                folder_id=1,
            )

    def test_no_folder_id_skips_sync(self, settings) -> None:
        """Should skip sync if folder_id is None."""
        settings.DJ_BREVO["AUTO_SYNC"] = True

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            BrevoList.objects.create(name="Test List")  # No folder_id

            mock_client.return_value.create_list.assert_not_called()


@pytest.mark.django_db
class TestBrevoAttributeSignal:
    """Tests for BrevoAttribute post_save signal."""

    def test_auto_sync_disabled_skips_api_call(self, settings) -> None:
        """Should not call API when AUTO_SYNC is False."""
        settings.DJ_BREVO["AUTO_SYNC"] = False  # Explicitly set

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            BrevoAttribute.objects.create(name="TEST_ATTR")

            mock_client.assert_not_called()

    def test_auto_sync_creates_attribute_in_brevo(self, settings) -> None:
        """Should call create_attribute and mark as synced."""
        settings.DJ_BREVO["AUTO_SYNC"] = True

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            mock_instance = mock_client.return_value

            attr = BrevoAttribute.objects.create(
                name="MRR",
                attribute_type="float",
                category="normal",
            )

            mock_instance.create_attribute.assert_called_once()

            # Refresh and check synced
            attr.refresh_from_db()
            assert attr.brevo_synced is True
            assert attr.synced_at is not None

    def test_already_synced_skips_api_call(self, settings) -> None:
        """Should not call API if already synced."""
        settings.DJ_BREVO["AUTO_SYNC"] = True

        with patch("dj_brevo.signals.handlers.BrevoClient") as mock_client:
            mock_instance = mock_client.return_value

            # Create (will sync)
            attr = BrevoAttribute.objects.create(name="TEST_ATTR")

            # Reset and save again
            mock_instance.reset_mock()
            attr.save()

            # Should not call API again
            mock_instance.create_attribute.assert_not_called()



from typing import Any, Literal, cast

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils import timezone

from dj_brevo.models import BrevoAttribute, BrevoList
from dj_brevo.services import BrevoClient
from dj_brevo.settings import brevo_settings

# Type aliases for Brevo API Literal types
AttributeCategory = Literal[
    "normal", "transactional", "category", "calculated", "global"
]
AttributeType = Literal[
    "text", "date", "float", "boolean", "id", "category", "multiple-choice", "user"
]


@receiver(post_save, sender=BrevoList)
def sync_brevo_list(
    sender: type[BrevoList],
    instance: BrevoList,
    created: bool,
    **kwargs: Any,
) -> None:
    """Trigger sync BrevoList >> Brevo on save."""
    # Skip if auto-sync is disabled
    if not brevo_settings.AUTO_SYNC:
        return

    # Skip if already syncing
    if getattr(instance, "_syncing", False):
        return

    client = BrevoClient()

    if instance.brevo_id is None:
        # Create new list in Brevo
        if instance.folder_id is None:
            return

        response = client.create_list(
            name=instance.name,
            folder_id=instance.folder_id,
        )

        # Save the returned brevo_id
        instance._syncing = True
        instance.brevo_id = response["id"]
        instance.synced_at = timezone.now()
        instance.save(update_fields=["brevo_id", "synced_at"])
        instance._syncing = False
    else:
        # Update existing list in Brevo
        client.update_list(
            list_id=instance.brevo_id,
            name=instance.name,
            folder_id=instance.folder_id,
        )
        instance._syncing = True
        instance.synced_at = timezone.now()
        instance.save(update_fields=["synced_at"])
        instance._syncing = False


@receiver(post_save, sender=BrevoAttribute)
def sync_brevo_attribute(
    sender: type[BrevoAttribute],
    instance: BrevoAttribute,
    created: bool,
    **kwargs: Any,
) -> None:
    """Sync BrevoAttribute >> Brevo on save"""
    # Skip if auto-sync is disabled
    if not brevo_settings.AUTO_SYNC:
        return

    # Skip if already synced to Brevo
    if instance.brevo_synced:
        return

    # Skip if already syncing
    if getattr(instance, "_syncing", False):
        return

    client = BrevoClient()

    # Build enumeration from options if category type
    enumeration = None
    if instance.attribute_type == "category":
        enumeration = [
            {"value": opt.value, "label": opt.label} for opt in instance.options.all()
        ]

    # Build multi_category_options if multiple-choice type
    multi_category_options = None
    if instance.attribute_type == "multiple-choice":
        multi_category_options = list(instance.options.values_list("label", flat=True))

    client.create_attribute(
        attribute_name=instance.name,
        attribute_category=cast(AttributeCategory, instance.category),
        attribute_type=cast(AttributeType, instance.attribute_type),
        enumeration=enumeration,
        multi_category_options=multi_category_options,
    )

    # Mark as synced
    instance._syncing = True
    instance.brevo_synced = True
    instance.synced_at = timezone.now()
    instance.save(update_fields=["brevo_synced", "synced_at"])
    instance._syncing = False



"""Base classes for dj-brevo models."""

from django.db import models


class BrevoSyncMixin(models.Model):
    """Abstract mixin for models that sync with Brevo.

    Provides common fields for tracking sync state.
    """

    synced_at = models.DateTimeField(
        null=True, blank=True, help_text="When this record was last synced with Brevo."
    )
    _syncing: bool = False

    class Meta:
        abstract = True



"""Django email backend for Brevo."""

from collections.abc import Sequence

from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.message import EmailMessage

from dj_brevo.services import BrevoClient


class BrevoEmailBackend(BaseEmailBackend):
    """Django email backend that sends emails via Brevo API.

    Usage:
    # settings.py
    EMAIL_BACKEND = "dj_brevo.backends.BrevoEmailBackend"

    # Then user Django's normal email functions:
    from django.core.mail import send_mail
    send_mail("Subject", "Body", "from@example.com", ["to@example.com"])
    """

    def __init__(self, api_key: str | None = None, fail_silently: bool = False) -> None:
        """Initialize the backend.

        Args:
            api_key: Optional Brevo API Key (defaults to settings).
            fail_silently: If True, suppress exceptions on send failure.
        """
        super().__init__(fail_silently=fail_silently)
        self.api_key = api_key
        self._client: BrevoClient | None = None

    @property
    def client(self) -> BrevoClient:
        """Lazy load Brevo client."""
        if self._client is None:
            self._client = BrevoClient(api_key=self.api_key)
        return self._client

    def send_messages(self, email_messages: Sequence[EmailMessage]) -> int:
        """Send one or more EmailMessage objects via Brevo.

        Args:
            email_messages: List of Django EmailMessage objects.

        Returns:
            Number of messages sent successfully.
        """
        if not email_messages:
            return 0

        sent_count = 0

        for message in email_messages:
            try:
                if self._send_message(message):
                    sent_count += 1
            except Exception:
                if not self.fail_silently:
                    raise

        return sent_count

    def _send_message(self, message: EmailMessage) -> bool:
        """Send a single EmailMessage via Brevo.

        Args:
            message: Django EmailMessage object.

        Returns:
            True if sent successfully.
        """
        # Convert Django's recipient format to Brevo's format
        to = [{"email": email} for email in message.to]
        cc = [{"email": email} for email in message.cc] if message.cc else None
        bcc = [{"email": email} for email in message.bcc] if message.bcc else None
        reply_to = {"email": message.reply_to[0]} if message.reply_to else None

        # Build sender dict
        sender = {"email": message.from_email} if message.from_email else None

        # Get HTML content if available
        html_content = self._get_html_content(message)

        # Send via our client
        self.client.send_email(
            to=to,
            subject=str(message.subject),
            html_content=html_content or f"<pre>{message.body}</pre>",
            text_content=str(message.body),
            sender=sender,
            cc=cc,
            bcc=bcc,
            reply_to=reply_to,
        )

        return True

    def _get_html_content(self, message: EmailMessage) -> str | None:
        """Extract HTML content from an EmailMessage.

        Args:
            message: Django EmailMessage object.

        Returns:
            HTML string if available, None otherwise.
        """
        # Check if this is an EmailMultiAlternatives with HTML
        alternatives = getattr(message, "alternatives", [])

        for content, mimetype in alternatives:
            if mimetype == "text/html":
                return str(content)

        return None



"""Minimal Django settings for testing"""

SECRET_KEY = "super-secret-key-not-for-production"

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "dj_brevo",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

USE_TZ = True

DJ_BREVO = {
    "API_KEY": "test-api-key",
    "DEFAULT_FROM_EMAIL": "test@example.com",
    "AUTO_SYNC": False,
}



from django.db import models

from .base import BrevoSyncMixin


class BrevoAttribute(BrevoSyncMixin):
    class AttributeType(models.TextChoices):
        TEXT = "text", "Text"
        FLOAT = "float", "Float"
        DATE = "date", "Date"
        BOOLEAN = "boolean", "Boolean"
        CATEGORY = "category", "Category"
        MULTIPLE_CHOICE = "multiple-choice", "Multiple Choice"
        USER = "user", "User"

    class Category(models.TextChoices):
        NORMAL = "normal", "Normal"
        CATEGORY = "category", "Category"
        TRANSACTIONAL = "transactional", "Transactional"

    name = models.CharField(max_length=255, unique=True)
    attribute_type = models.CharField(
        max_length=20,
        choices=AttributeType.choices,
        default=AttributeType.TEXT,
    )
    category = models.CharField(
        max_length=20,
        choices=Category.choices,
        default=Category.NORMAL,
    )
    brevo_synced = models.BooleanField(
        default=False, help_text="If this attribute has been created in Brevo."
    )

    class Meta:
        verbose_name = "Brevo Attribute"
        verbose_name_plural = "Brevo Attributes"

    def __str__(self) -> str:
        return self.name


class BrevoAttributeOption(models.Model):
    attribute = models.ForeignKey(
        BrevoAttribute,
        on_delete=models.CASCADE,
        related_name="options",
    )
    value = models.IntegerField(
        null=True, blank=True, help_text="Numeric value for category-type attributes."
    )
    label = models.CharField(max_length=200)
    order = models.PositiveIntegerField(default=0)

    class Meta:
        verbose_name = "Brevo Attribute Option"
        verbose_name_plural = "Brevo Attribute Options"
        unique_together = [("attribute", "label")]
        ordering = ["order"]

    def __str__(self) -> str:
        return self.label
